@@ -3,7 +3,7 @@
  Documentation generator for the Lua/APR binding.
 
  Author: Peter Odding <peter@peterodding.com>
- Last Change: July 3, 2011
+ Last Change: December 6, 2011
  Homepage: http://peterodding.com/code/lua/apr/
  License: MIT
 
@@ -28,6 +28,7 @@ local SOURCES = [[
   memcache.c
   getopt.c
   http.c
+  pollset.c
   proc.c
   shm.c
   signal.c
@@ -40,6 +41,7 @@ local SOURCES = [[
   uuid.c
   xlate.c
   xml.c
+  serialize.c
   apr.lua
   lua_apr.c
   permissions.c
@@ -47,6 +49,7 @@ local SOURCES = [[
   ../examples/download.lua
   ../examples/webserver.lua
   ../examples/threaded-webserver.lua
+  ../examples/async-webserver.lua
 ]]
 
 local modules = {}
@@ -395,6 +398,8 @@ local bsignore = {
   ['apr.parse_headers'] = true,
   ['apr.parse_multipart'] = true,
   ['apr.parse_query_string'] = true,
+  ['apr.ref'] = true,
+  ['apr.deref'] = true,
   ['apr.platform_get'] = true,
   ['apr.proc_fork'] = true,
   ['apr.signal_names'] = true,
@@ -428,6 +433,7 @@ local bsignore = {
   ['mc_client:version'] = true,
   ['md5_context:digest'] = true,
   ['sha1_context:digest'] = true,
+  ['socket:recvfrom'] = true,
   ['socket:listen'] = true,
   ['thread:join'] = true,
   ['thread:status'] = true,
@@ -438,17 +444,22 @@ local function dumpentries(functions)
     local signature = entry.signature:gsub('%->', '→')
     local funcname = sig2pubfun(signature)
     local anchor = toanchor(funcname)
-    local covkey = sig2privfun(signature)
-    if not coverage[covkey] then covkey = 'lua_' .. covkey end
-    local tc = coverage[covkey] or ''
-    if tc ~= '' then
-      local template = '<span style="float: right; font-size: small; color: %s; opacity: 0.5">test coverage: %s<br></span>'
-      local color = tc >= 75 and 'green' or tc >= 50 and 'orange' or 'red'
-      tc = fmt(template, color, tc == 0 and 'none' or fmt('%i%%', tc))
+    local tc = ''
+    if next(coverage) then
+      local covkey = sig2privfun(signature)
+      if not coverage[covkey] then covkey = 'lua_' .. covkey end
+      tc = coverage[covkey] or ''
+      if tc ~= '' then
+        local template = '<span style="float: right; font-size: small; color: %s; opacity: 0.5">test coverage: %s<br></span>'
+        local color = tc >= 75 and 'green' or tc >= 50 and 'orange' or 'red'
+        tc = fmt(template, color, tc == 0 and 'none' or fmt('%i%%', tc))
+      end
     end
     blocks:add('### %s <a name="%s" href="#%s">`%s`</a>', tc, anchor, anchor, signature)
     blocks:add('%s', preprocess(entry.description))
-    if entry.binarysafe ~= nil and not bsignore[funcname] then
+    if entry.binarysafe ~= nil and not bsignore[funcname]
+        and not entry.description:find 'This function is binary safe'
+        and not entry.description:find 'This function is not binary safe' then
       blocks:add('*This function %s binary safe.*', entry.binarysafe and 'is' or 'is not')
     end
   end
@@ -465,8 +476,14 @@ local custom_sorting = {
     apr.password_get apr.md5_init md5_context:update md5_context:digest
     md5_context:reset apr.sha1 apr.sha1_init sha1_context:update
     sha1_context:digest sha1_context:reset ]],
-  ['thread.c'] = [[ apr.thread apr.thread_create apr.thread_yield
-    thread:status thread:join ]],
+  ['thread.c'] = [[ apr.thread apr.thread_yield thread:status thread:join ]],
+  ['serialize.c'] = [[ apr.serialize apr.unserialize apr.ref apr.deref ]],
+  ['io_file.c'] = [[ apr.file_link apr.file_copy apr.file_append
+    apr.file_rename apr.file_remove apr.file_truncate apr.file_mtime_set
+    apr.file_attrs_set apr.file_perms_set apr.stat apr.file_open file:stat
+    file:lines file:truncate file:read file:write file:seek file:flush
+    file:lock file:unlock pipe:timeout_get pipe:timeout_set file:fd_get
+    file:inherit_set file:inherit_unset file:close ]],
 }
 
 for _, module in ipairs(sorted_modules) do
