@@ -3,7 +3,7 @@
  Lua source code for the Lua/APR binding.
 
  Author: Peter Odding <peter@peterodding.com>
- Last Change: July 3, 2011
+ Last Change: December 7, 2011
  Homepage: http://peterodding.com/code/lua/apr/
  License: MIT
 
@@ -14,7 +14,18 @@
 --]]
 
 local apr = require 'apr.core'
-apr._VERSION = '0.20'
+
+apr._VERSION = '0.23.2'
+
+local function executable(type, user, group, protection)
+  if type == 'file' and user and group and protection then
+    local current_user, current_group = assert(apr.user_get())
+    return (protection:find '^(r.x)(...)(...)$' and user == current_user)
+        or (protection:find '^(...)(r.x)(...)$' and group == current_group)
+        or (protection:find '^(...)(...)(r.[xt])$' and user ~= current_user and group ~= current_group)
+  end
+  return false
+end
 
 -- apr.md5(input [, binary]) -> digest {{{1
 --
@@ -66,6 +77,25 @@ function apr.sha1(input, binary)
   return nil, errmsg, errcode
 end
 
+-- apr.file_truncate(path [, offset]) -> status {{{1
+--
+-- Truncate the file's length to the specified @offset (defaults to 0). On
+-- success true is returned, otherwise a nil followed by an error message is
+-- returned.
+--
+-- Part of the "File I/O handling" module.
+
+function apr.file_truncate(path, offset)
+  local status = nil
+  local handle, errmsg, errcode = apr.file_open(path, 'r+')
+  if handle then
+    status, errmsg, errcode = handle:truncate(offset or 0)
+    if status then return handle:close() end
+    handle:close()
+  end
+  return status, errmsg, errcode
+end
+
 -- apr.filepath_which(program [, find_all]) -> pathname {{{1
 --
 -- Find the full pathname of @program by searching the directories in the
@@ -84,14 +114,14 @@ function apr.filepath_which(program, find_all)
   local results = find_all and {}
   for _, directory in ipairs(split(apr.env_get 'PATH')) do
     local candidate = apr.filepath_merge(directory, program)
-    if apr.stat(candidate, 'type') == 'file' then
-      -- TODO if not is_windows check executable bits
+    local t, u, g, p = apr.stat(candidate, 'type', 'user', 'group', 'protection')
+    if t == 'file' and is_windows or executable(t, u, g, p) then
       if not find_all then return candidate end
       results[#results + 1] = candidate
     end
     if is_windows and #extensions >= 1 then
       for _, extension in ipairs(extensions) do
-        candidate = apr.filepath_merge(directory, program .. '.' .. extension)
+        candidate = apr.filepath_merge(directory, program .. extension)
         if apr.stat(candidate, 'type') == 'file' then
           if not find_all then return candidate end
           results[#results + 1] = candidate
@@ -102,6 +132,17 @@ function apr.filepath_which(program, find_all)
   return results
 end
 
+-- apr.filepath_executable(path) -> is_executable {{{1
+--
+-- Check whether the file pointed to by @path is executable.
+-- Returns true when the file is executable, false otherwise.
+--
+-- Part of the "File path manipulation" module.
+
+function apr.filepath_executable(path)
+  return executable(apr.stat(path, 'type', 'user', 'group', 'protection'))
+end
+
 -- apr.glob(pattern [, ignorecase]) -> iterator {{{1
 --
 -- Split @pattern into a directory path and a filename pattern and return an
@@ -279,25 +320,27 @@ function apr.getopt(usage, config)
   end
 end
 
-if apr.thread_create then
-
--- apr.thread(body [, arg, ...]) -> thread {{{1
+-- apr.serialize(...) -> string {{{1
 --
--- Shortcut for `apr.thread_create()` that supports actual functions as @body
--- by converting them to byte code using `string.dump()`. This means you don't
--- lose syntax highlighting and debug information. This also calls `assert()`
--- on the results of `apr.thread_create()`. This function won't work in Lua
--- implementations like LuaJIT 2 where `string.dump()` is not available.
+-- Serialize any number of Lua values (a tuple) into a source code string. When
+-- passed to `apr.unserialize()` this string results in a tuple of values that
+-- is structurally identical to the original tuple.
 --
--- Part of the "Multi threading" module.
+-- Part of the "Serialization" module.
 
-function apr.thread(body, ...)
-  if type(body) == 'function' then
-    body = string.dump(body)
-  end
-  return assert(apr.thread_create(body, ...))
+function apr.serialize(...)
+  return require 'apr.serialize' ({ n = select('#', ...), ... })
 end
 
+-- apr.unserialize(string) -> ... {{{1
+--
+-- Unserialize a source code string into one or more Lua values.
+--
+-- Part of the "Serialization" module.
+
+function apr.unserialize(data)
+  local tuple = loadstring(data)()
+  return unpack(tuple, 1, tuple.n)
 end
 
 -- }}}1
