@@ -3,7 +3,7 @@
  * Authors:
  *  - zhiguo zhao <zhaozg@gmail.com>
  *  - Peter Odding <peter@peterodding.com>
- * Last Change: June 30, 2011
+ * Last Change: November 6, 2011
  * Homepage: http://peterodding.com/code/lua/apr/
  * License: MIT
  *
@@ -71,6 +71,10 @@
 #include "lua_apr.h"
 #include <apr_dbd.h>
 
+/* Enable compatibility with APR 1.2? (which doesn't
+ * have named columns and explicit transaction modes) */
+#define LUA_APR_DBD_COMPAT (APR_MAJOR_VERSION == 1 && APR_MINOR_VERSION <= 2)
+
 /* Lua object structures. {{{1 */
 
 /* Driver objects. */
@@ -109,6 +113,7 @@ typedef struct {
   lua_apr_refobj header;
   lua_apr_dbd_reference parent;
   apr_dbd_prepared_t *statement;
+  int random_access;
 } lua_apr_dbp_object;
 
 /* Internal functions. {{{1 */
@@ -189,9 +194,13 @@ static lua_apr_dbr_object *new_resultset(lua_State *L, int idx, lua_apr_dbd_obje
 
 /* new_statement() {{{2 */
 
-static lua_apr_dbp_object *new_statement(lua_State *L, int idx, lua_apr_dbd_object *driver)
+static lua_apr_dbp_object *new_statement(lua_State *L, int idx, lua_apr_dbd_object *driver, int random_access)
 {
-  return new_subobj(L, &lua_apr_dbp_type, idx, driver, offsetof(lua_apr_dbp_object, parent));
+  lua_apr_dbp_object *statement;
+
+  statement = new_subobj(L, &lua_apr_dbp_type, idx, driver, offsetof(lua_apr_dbp_object, parent));
+  statement->random_access = random_access;
+  return statement;
 }
 
 /* check_dbd_value() {{{2 */
@@ -359,6 +368,8 @@ static int dbr_tuple_cb(lua_State *L, lua_apr_dbr_object *results, apr_dbd_row_t
 
 /* dbr_row_cb() {{{2 */
 
+#if !LUA_APR_DBD_COMPAT
+
 static int dbr_row_cb(lua_State *L, lua_apr_dbr_object *results, apr_dbd_row_t *row)
 {
   lua_apr_dbd_object *driver;
@@ -377,6 +388,8 @@ static int dbr_row_cb(lua_State *L, lua_apr_dbr_object *results, apr_dbd_row_t *
   return is_pairs ? 2 : 1;
 }
 
+#endif
+
 /* dbd_close_impl() {{{2 */
 
 static apr_status_t dbd_close_impl(lua_apr_dbd_object *driver)
@@ -494,7 +507,11 @@ int lua_apr_dbd(lua_State *L)
 static int dbd_open(lua_State *L)
 {
   lua_apr_dbd_object *driver;
+# if LUA_APR_DBD_COMPAT
+  const char *params;
+# else
   const char *params, *error;
+# endif
   apr_status_t status;
 
   driver = check_dbd(L, 1, 0, 0);
@@ -510,13 +527,21 @@ static int dbd_open(lua_State *L)
     driver->generation++;
     driver->handle = NULL;
   }
+# if LUA_APR_DBD_COMPAT
+  status = apr_dbd_open(driver->driver, driver->pool, params,
+      &driver->handle);
+  if (status != APR_SUCCESS)
+    return push_error_status(L, status);
+# else
   status = apr_dbd_open_ex(driver->driver, driver->pool, params,
       &driver->handle, &error);
   if (status != APR_SUCCESS) {
     lua_pushnil(L);
     lua_pushstring(L, error);
-    return 2;
+    status_to_name(L, status);
+    return 3;
   }
+# endif
   lua_pushboolean(L, 1);
   return 1;
 }
@@ -632,6 +657,8 @@ static int dbd_select(lua_State *L)
  * inherit the error code and any further query/select calls will fail
  * immediately. Put transaction in `'ignore-errors'` mode to avoid that. Use
  * `'rollback'` mode to do explicit rollback.
+ *
+ * TODO Support for transaction objects that have query(), select(), prepare() methods?
  */
 
 static int dbd_transaction_start(lua_State *L)
@@ -664,6 +691,8 @@ static int dbd_transaction_end(lua_State *L)
   return push_status(L, status);
 }
 
+#if !LUA_APR_DBD_COMPAT
+
 /* driver:transaction_mode([mode]) -> mode {{{2
  *
  * Get or set the transaction mode, one of:
@@ -705,12 +734,17 @@ static int dbd_transaction_mode(lua_State *L)
   return 0;
 }
 
+#endif
+
 /* Prepared statements. {{{1 */
 
-/* driver:prepare(sql) -> prepared_statement {{{2
+/* driver:prepare(sql [, random_access]) -> prepared_statement {{{2
  *
- * Prepare a statement. On success a prepared statement object is returned,
- * otherwise a nil followed by an error message is returned.
+ * Prepare an SQL statement. On success a prepared statement object is
+ * returned, otherwise a nil followed by an error message is returned. The
+ * string @sql gives the query to prepare. If the optional argument
+ * @random_access is true, result sets created by the prepared statement will
+ * support random access.
  *
  * To specify parameters of the prepared query, use `%s`, `%d` etc. (see below
  * for full list) in place of database specific parameter syntax (e.g. for
@@ -748,10 +782,12 @@ static int dbd_prepare(lua_State *L)
   lua_apr_dbp_object *statement;
   const char *query;
   apr_status_t status;
+  int random_access;
 
   driver = check_dbd(L, 1, 1, 0);
   query = luaL_checkstring(L, 2);
-  statement = new_statement(L, 1, driver);
+  random_access = lua_toboolean(L, 3);
+  statement = new_statement(L, 1, driver, random_access);
   status = apr_dbd_prepare(driver->driver, driver->pool, driver->handle,
       query, NULL, &statement->statement);
   if (status != APR_SUCCESS)
@@ -789,7 +825,7 @@ static int dbp_query(lua_State *L)
   return push_query_result(L, driver, nrows, status);
 }
 
-/* prepared_statement:select(random_access, ...) -> result_set {{{2
+/* prepared_statement:select(...) -> result_set {{{2
  *
  * Using a prepared statement, execute an SQL query that returns a result set.
  * On success a result set object is returned, otherwise a nil followed by an
@@ -800,9 +836,6 @@ static int dbp_query(lua_State *L)
  * otherwise all function arguments after @random_access become query
  * parameters.
  *
- * TODO Make the @random_access argument an attribute of the prepared
- * statement object so that we can get rid of the argument here?!
- *
  * *This function is not binary safe.*
  */
 
@@ -812,15 +845,14 @@ static int dbp_select(lua_State *L)
   lua_apr_dbp_object *statement;
   lua_apr_dbr_object *results;
   const char **values;
-  int nargs, status, random_access;
+  int nargs, status;
 
   statement = check_dbp(L, 1);
-  random_access = lua_toboolean(L, 2);
   driver = statement->parent.driver;
-  check_dbd_values(L, 3, &nargs, &values);
-  results = new_resultset(L, 1, driver, random_access);
+  check_dbd_values(L, 2, &nargs, &values);
+  results = new_resultset(L, 1, driver, statement->random_access);
   status = apr_dbd_pselect(driver->driver, driver->pool, driver->handle,
-      &results->results, statement->statement, random_access, nargs, values);
+      &results->results, statement->statement, statement->random_access, nargs, values);
   free((void*) values); /* make MSVC++ 2010 happy */
   if (status != 0)
     return push_dbd_error(L, driver, status);
@@ -851,6 +883,8 @@ static int dbp_gc(lua_State *L)
 
 /* Result sets. {{{1 */
 
+#if !LUA_APR_DBD_COMPAT
+
 /* result_set:columns([num]) -> name [, ...] {{{2
  *
  * If no arguments are given return the names of all columns in the result set,
@@ -923,6 +957,8 @@ static int dbr_rows(lua_State *L)
   return 1;
 }
 
+#endif
+
 /* result_set:tuple([rownum]) -> value [, ...] {{{2
  *
  * Return a tuple for the next row in the result set or the row with index
@@ -966,6 +1002,8 @@ static int dbr_tuples(lua_State *L)
   return 1;
 }
 
+#if !LUA_APR_DBD_COMPAT
+
 /* result_set:pairs() -> iterator {{{2
  *
  * Return an iterator that produces a row number and a table with named fields
@@ -986,6 +1024,8 @@ static int dbr_pairs(lua_State *L)
   return 1;
 }
 
+#endif
+
 /* #result_set -> num_tuples {{{2
  *
  * Get the number of rows in a result set of a synchronous select. If the
@@ -1094,7 +1134,9 @@ static luaL_reg dbd_methods[] = {
   { "close", dbd_close },
   { "transaction_start", dbd_transaction_start },
   { "transaction_end", dbd_transaction_end },
+# if !LUA_APR_DBD_COMPAT
   { "transaction_mode", dbd_transaction_mode },
+# endif
   { NULL, NULL }
 };
 
@@ -1113,20 +1155,24 @@ static luaL_reg dbr_metamethods[] = {
   { "__tostring", dbr_tostring },
   { "__eq", objects_equal },
   { "__gc", dbr_gc },
-#if LUA_VERSION_NUM >= 502
+# if LUA_VERSION_NUM >= 502 && !LUA_APR_DBD_COMPAT
   { "__pairs", dbr_rows },
   { "__ipairs", dbr_pairs },
-#endif
+# endif
   { NULL, NULL }
 };
 
 static luaL_reg dbr_methods[] = {
+# if !LUA_APR_DBD_COMPAT
   { "columns", dbr_columns },
+# endif
   { "tuple", dbr_tuple },
   { "tuples", dbr_tuples },
+# if !LUA_APR_DBD_COMPAT
   { "row", dbr_row },
   { "rows", dbr_rows },
   { "pairs", dbr_pairs },
+# endif
   { NULL, NULL }
 };
 
