@@ -1,7 +1,7 @@
 /* DBM routines module for the Lua/APR binding.
  *
  * Author: Peter Odding <peter@peterodding.com>
- * Last Change: February 19, 2011
+ * Last Change: June 16, 2011
  * Homepage: http://peterodding.com/code/lua/apr/
  * License: MIT
  *
@@ -332,7 +332,7 @@ int dbm_gc(lua_State *L)
   lua_apr_dbm *dbm = dbm_check(L, 1, 0);
   if (object_collectable((lua_apr_refobj*)dbm))
     dbm_close_impl(L, dbm);
-  release_object(L, (lua_apr_refobj*)dbm);
+  release_object((lua_apr_refobj*)dbm);
   return 0;
 }
 
