@@ -1,9 +1,17 @@
 /* Network I/O handling module for the Lua/APR binding.
  *
  * Author: Peter Odding <peter@peterodding.com>
- * Last Change: June 24, 2011
+ * Last Change: December 7, 2011
  * Homepage: http://peterodding.com/code/lua/apr/
  * License: MIT
+ *
+ * To get started with network programming using the Lua/APR binding, have a
+ * look at any of the following examples:
+ *
+ *  - [HTTP client](#example_http_client)
+ *  - [Single threaded webserver](#example_single_threaded_webserver)
+ *  - [Multi threaded webserver](#example_multi_threaded_webserver)
+ *  - [Asynchronous webserver](#example_asynchronous_webserver)
  */
 
 #include "lua_apr.h"
@@ -12,15 +20,18 @@
 
 /* Internal functions {{{1 */
 
-/* Socket object structure declaration. */
-typedef struct {
-  lua_apr_refobj header;
-  lua_apr_readbuf input;
-  lua_apr_writebuf output;
-  apr_pool_t *pool;
-  apr_socket_t *handle;
-  int family, protocol;
-} lua_apr_socket;
+/* family_check(L, i) -- check for address family on Lua stack {{{2 */
+
+#if APR_HAVE_IPV6
+const char *family_options[] = { "inet", "inet6", "unspec", NULL };
+const int family_values[] = { APR_INET, APR_INET6, APR_UNSPEC };
+#else
+const char *family_options[] = { "inet", "unspec", NULL };
+const int family_values[] = { APR_INET, APR_UNSPEC };
+#endif
+
+#define family_check(L, i) \
+  family_values[luaL_checkoption(L, i, "inet", family_options)]
 
 /* socket_alloc(L) -- allocate and initialize socket object {{{2 */
 
@@ -58,20 +69,6 @@ static lua_apr_socket* socket_check(lua_State *L, int i, int open)
   return object;
 }
 
-/* family_check(L, i) -- check for address family on Lua stack {{{2 */
-
-static int family_check(lua_State *L, int i)
-{
-# if APR_HAVE_IPV6
-  const char *options[] = { "inet", "inet6", "unspec", NULL };
-  const int values[] = { APR_INET, APR_INET6, APR_UNSPEC };
-# else
-  const char *options[] = { "inet", "unspec", NULL };
-  const int values[] = { APR_INET, APR_UNSPEC };
-# endif
-  return values[luaL_checkoption(L, i, "inet", options)];
-}
-
 /* option_check(L, i) -- check for socket option on Lua stack {{{2 */
 
 static apr_int32_t option_check(lua_State *L, int i)
@@ -173,14 +170,19 @@ int lua_apr_hostname_get(lua_State *L)
   return 1;
 }
 
-/* apr.host_to_addr(hostname [, family]) -> ip_address {{{1
+/* apr.host_to_addr(hostname [, family]) -> ip_address, ... {{{1
  *
- * Resolve a host name to an IP-address. On success the IP-address is returned
- * as a string, otherwise a nil followed by an error message is returned. The
- * optional @family argument is documented under `apr.socket_create()`.
+ * Resolve a host name to one or more IP addresses. On success one or more IP
+ * addresses are returned as strings, otherwise a nil followed by an error
+ * message is returned. The optional @family argument is documented under
+ * `apr.socket_create()`.
  *
  *     > = apr.host_to_addr 'www.lua.org'
  *     '89.238.129.35'
+ *     > = apr.host_to_addr 'google.com'
+ *     '173.194.65.104' '173.194.65.106' '173.194.65.147' '173.194.65.103' '173.194.65.105' '173.194.65.99'
+ *     > = apr.host_to_addr('ipv6.google.com', 'inet6')
+ *     '2a00:1450:8005::67'
  */
 
 int lua_apr_host_to_addr(lua_State *L)
@@ -195,14 +197,22 @@ int lua_apr_host_to_addr(lua_State *L)
   pool = to_pool(L);
   host = luaL_checkstring(L, 1);
   family = family_check(L, 2);
+
   status = apr_sockaddr_info_get(&address, host, family, SOCK_STREAM, 0, pool);
-  if (status == APR_SUCCESS)
-    status = apr_sockaddr_ip_get(&ip_address, address);
   if (status != APR_SUCCESS)
     return push_error_status(L, status);
-  lua_pushstring(L, ip_address);
 
-  return 1;
+  lua_settop(L, 0);
+
+  do {
+    status = apr_sockaddr_ip_get(&ip_address, address);
+    if (status != APR_SUCCESS)
+      return push_error_status(L, status);
+    lua_pushstring(L, ip_address);
+    address = address->next;
+  } while (address != NULL);
+
+  return lua_gettop(L);
 }
 
 /* apr.addr_to_host(ip_address [, family]) -> hostname {{{1
@@ -343,6 +353,77 @@ static int socket_listen(lua_State *L)
   return push_status(L, status);
 }
 
+/* socket:recvfrom([bufsize]) -> address, data {{{1
+ *
+ * Read data from an [UDP] [udp] socket that has been bound to an interface
+ * and/or port. On success two values are returned: A table with the address of
+ * the peer from which the data was sent and a string with the received data.
+ * If the call fails it returns nil followed by an error message.
+ *
+ * By default @bufsize is 1024, this means the resulting @data string will be
+ * truncated to a maximum of 1024 bytes. If you want to receive larger
+ * messages, pass a larger @bufsize.
+ *
+ * The returned @address table contains the following fields:
+ *
+ *  - `address` is the IP address (a string)
+ *  - `port` is the port number
+ *  - `family` is one of the strings `'inet'`, `'inet6'` or `'unspec'`
+ *
+ * *This function is binary safe.*
+ *
+ * [udp]: http://en.wikipedia.org/wiki/User_Datagram_Protocol
+ */
+
+static int socket_recvfrom(lua_State *L)
+{
+  lua_apr_socket *object;
+  apr_status_t status;
+  apr_sockaddr_t address = { 0 };
+  apr_size_t buflen;
+  apr_int32_t flags;
+  char buffer[1024], ip_addr[APRMAXHOSTLEN];
+  char *bufptr;
+  int i;
+
+  /* Validate arguments. */
+  object = socket_check(L, 1, 1);
+  buflen = luaL_optint(L, 2, sizeof buffer);
+
+  /* Use dynamically allocated buffer only when necessary. */
+  bufptr = (buflen > sizeof buffer) ? lua_newuserdata(L, buflen) : &buffer[0];
+
+  flags = 0;
+  status = apr_socket_recvfrom(&address, object->handle, flags, bufptr, &buflen);
+  if (status != APR_SUCCESS)
+    return push_error_status(L, status);
+
+  /* Convert the socket address to a Lua table. */
+  lua_newtable(L);
+
+  /* Get the IP address in numeric format. */
+  status = apr_sockaddr_ip_getbuf(ip_addr, sizeof ip_addr, &address);
+  if (status == APR_SUCCESS)
+    lua_pushstring(L, ip_addr), lua_setfield(L, -2, "address");
+
+  /* Get the port number. */
+  lua_pushnumber(L, address.port), lua_setfield(L, -2, "port");
+
+  /* Get the address family. */
+  for (i = 0; i < count(family_values); i++)
+    if (family_values[i] == address.family) {
+      lua_pushstring(L, family_options[i]);
+      lua_setfield(L, -2, "family");
+      break;
+    }
+
+  /* Push the received data. */
+  lua_pushlstring(L, bufptr, buflen);
+
+  /* Return address table and received data. */
+  return 2;
+}
+
 /* socket:accept() -> client_socket {{{1
  *
  * Accept a connection request on a server socket. On success a socket is
@@ -673,6 +754,7 @@ static int socket_gc(lua_State *L)
 luaL_reg socket_methods[] = {
   { "bind", socket_bind },
   { "listen", socket_listen },
+  { "recvfrom", socket_recvfrom },
   { "accept", socket_accept },
   { "connect", socket_connect },
   { "read", socket_read },
