@@ -1,17 +1,18 @@
 /* Miscellaneous functions module for the Lua/APR binding.
  *
  * Author: Peter Odding <peter@peterodding.com>
- * Last Change: July 3, 2011
+ * Last Change: November 20, 2011
  * Homepage: http://peterodding.com/code/lua/apr/
  * License: MIT
  */
 
 #include "lua_apr.h"
+#include <apr_ldap.h> /* APR_HAS_LDAP */
 #include <apr_portable.h>
+#include <ctype.h>
 #if LUA_APR_HAVE_APREQ
 #include <apreq_version.h>
 #endif
-#include <ctype.h>
 
 /* Used to make sure that APR is only initialized once. */
 static int apr_was_initialized = 0;
@@ -25,14 +26,19 @@ lua_apr_objtype *lua_apr_types[] = {
   &lua_apr_thread_type,
   &lua_apr_queue_type,
 # endif
+  &lua_apr_pollset_type,
   &lua_apr_proc_type,
   &lua_apr_dbm_type,
   &lua_apr_dbd_type,
   &lua_apr_dbr_type,
   &lua_apr_dbp_type,
+# if APR_HAS_LDAP
   &lua_apr_ldap_type,
+# endif
+#if LUA_APR_HAVE_MEMCACHE
   &lua_apr_memcache_type,
   &lua_apr_memcache_server_type,
+#endif
   &lua_apr_md5_type,
   &lua_apr_sha1_type,
   &lua_apr_xml_type,
@@ -54,6 +60,8 @@ LUA_APR_EXPORT int luaopen_apr_core(lua_State *L)
     { "os_default_encoding", lua_apr_os_default_encoding },
     { "os_locale_encoding", lua_apr_os_locale_encoding },
     { "type", lua_apr_type },
+    { "ref", lua_apr_ref },
+    { "deref", lua_apr_deref },
 
     /* base64.c -- base64 encoding/decoding. */
     { "base64_encode", lua_apr_base64_encode },
@@ -145,11 +153,16 @@ LUA_APR_EXPORT int luaopen_apr_core(lua_State *L)
     { "namedpipe_create", lua_apr_namedpipe_create },
     { "pipe_create", lua_apr_pipe_create },
 
+#   if APR_HAS_LDAP
     /* ldap.c -- LDAP connection handling. */
     { "ldap", lua_apr_ldap },
     { "ldap_info", lua_apr_ldap_info },
     { "ldap_url_check", lua_apr_ldap_url_check },
     { "ldap_url_parse", lua_apr_ldap_url_parse },
+#   endif
+
+    /* pollset -- asynchronous network communication. */
+    { "pollset", lua_apr_pollset },
 
     /* proc -- process handling. */
     { "proc_create", lua_apr_proc_create },
@@ -179,7 +192,7 @@ LUA_APR_EXPORT int luaopen_apr_core(lua_State *L)
 #   if APR_HAS_THREADS
 
     /* thread.c -- multi threading. */
-    { "thread_create", lua_apr_thread_create },
+    { "thread", lua_apr_thread },
     { "thread_yield", lua_apr_thread_yield },
 
     /* thread_queue.c -- thread queues. */
@@ -214,8 +227,10 @@ LUA_APR_EXPORT int luaopen_apr_core(lua_State *L)
     /* xml.c -- XML parsing. */
     { "xml", lua_apr_xml },
 
+#   if LUA_APR_HAVE_MEMCACHE
     /* memcache.c -- memcached client. */
     { "memcache", lua_apr_memcache },
+#   endif
 
     { NULL, NULL }
   };
