@@ -3,13 +3,17 @@
  Unit tests for the filename matching module of the Lua/APR binding.
 
  Author: Peter Odding <peter@peterodding.com>
- Last Change: February 11, 2011
+ Last Change: March 27, 2011
  Homepage: http://peterodding.com/code/lua/apr/
  License: MIT
 
 --]]
 
-local apr = require 'apr'
+local status, apr = pcall(require, 'apr')
+if not status then
+  pcall(require, 'luarocks.require')
+  apr = require 'apr'
+end
 
 -- Check that the ?, *, and [] wild cards are supported.
 assert(apr.fnmatch('lua_apr.?', 'lua_apr.c'))
