@@ -3,7 +3,7 @@
  Unit tests for the LDAP connection handling module of the Lua/APR binding.
 
  Author: Peter Odding <peter@peterodding.com>
- Last Change: July 2, 2011
+ Last Change: October 29, 2011
  Homepage: http://peterodding.com/code/lua/apr/
  License: MIT
 
@@ -17,4 +17,10 @@ if not status then
   apr = require 'apr'
 end
 local helpers = require 'apr.test.helpers'
+
+if not apr.ldap then
+  helpers.warning "LDAP module not available!\n"
+  return false
+end
+
 return helpers.ld_preload_trick 'ldap-child.lua'
