@@ -3,13 +3,17 @@
  Unit tests for the miscellaneous routines of the Lua/APR binding.
 
  Author: Peter Odding <peter@peterodding.com>
- Last Change: February 11, 2011
+ Last Change: July 1, 2011
  Homepage: http://peterodding.com/code/lua/apr/
  License: MIT
 
 --]]
 
-local apr = require 'apr'
+local status, apr = pcall(require, 'apr')
+if not status then
+  pcall(require, 'luarocks.require')
+  apr = require 'apr'
+end
 local helpers = require 'apr.test.helpers'
 
 -- Test apr.type()
@@ -20,9 +24,10 @@ assert(apr.type(apr.proc_create 'test') == 'process')
 assert(apr.type(apr.dir_open '.') == 'directory')
 
 -- Test apr.version_get()
-local apr_v, apu_v = assert(apr.version_get())
-assert(apr_v:find '^%d+%.%d+%.%d+$')
-assert(apu_v:find '^%d+%.%d+%.%d+$')
+local v = apr.version_get()
+assert(v.apr:find '^%d+%.%d+%.%d+$')
+assert(v.aprutil == nil or v.aprutil:find '^%d+%.%d+%.%d+$')
+assert(v.apreq == nil or v.apreq:find '^%d+%.%d+%.%d+$')
 
 -- Test status_to_name() (indirectly).
 assert(select(3, apr.stat("I assume this won't exist")) == 'ENOENT')
