@@ -3,7 +3,7 @@
  Unit tests for the thread queues module of the Lua/APR binding.
 
  Author: Peter Odding <peter@peterodding.com>
- Last Change: June 21, 2011
+ Last Change: November 20, 2011
  Homepage: http://peterodding.com/code/lua/apr/
  License: MIT
 
@@ -25,14 +25,14 @@ local queue = assert(apr.thread_queue(1))
 assert(not queue:trypop())
 
 -- Pass the thread queue to a thread.
-local thread = apr.thread(function(queue)
+local thread = assert(apr.thread(function()
   local status, apr = pcall(require, 'apr')
   if not status then
     pcall(require, 'luarocks.require')
     apr = require 'apr'
   end
   local helpers = require 'apr.test.helpers'
-  return helpers.try(function()
+  helpers.try(function()
     -- Scalar values.
     assert(queue:push(nil))
     assert(queue:push(false))
@@ -50,7 +50,7 @@ local thread = apr.thread(function(queue)
     helpers.message("Thread queue tests failed in child thread: %s\n", errmsg)
     assert(queue:terminate())
   end)
-end, queue)
+end))
 
 helpers.try(function()
   -- Check the sequence of supported value types.
